/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is wmlbrowser.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// Magic key with which we will find the 'res' directory of the
// browser installation
const resourceDir = "ARes";

// All the DTDs we want to download
const WMLDTDs = [ "http://www.wapforum.org/DTD/wml12.dtd",
                  "http://www.wapforum.org/DTD/wml_1_1.dtd",
                  "http://www.wapforum.org/DTD/wml_1.1.xml",
                  "http://www.wapforum.org/DTD/wml13.dtd" ];

// Return the res/dtd directory of the browser installation
function getDTDDirectory () {
    var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService();
    if (fileLocator)
        fileLocator = fileLocator.QueryInterface(Components.interfaces.nsIProperties);
    var file = fileLocator.get(resourceDir, Components.interfaces.nsIFile);
    file.append ("dtd");
    return file;
}

// Are all of the required DTDs correctly installed?
function areDTDsInstalled () {
    var logger = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
    try {
        var file = getDTDDirectory();
        var entries = file.directoryEntries;
        var dtdCount = 0;
        while (entries.hasMoreElements()) {
            var entry = entries.getNext();
            entry = entry.QueryInterface (Components.interfaces.nsIFile);
            for (var ix = 0; ix < WMLDTDs.length; ix++) {
                var knownDTD = WMLDTDs[ix];
                knownDTD = knownDTD.match(/[^\/]+$/);
                if (entry.leafName == knownDTD) {
                    dtdCount++;
                }
            }
        }
        return (dtdCount == WMLDTDs.length);
    } catch (e) {
        logger.logStringMessage(e);
    }
}

// We will produce an array of objects responsible for downloading the DTDs
var downloaders = [];

function downloadDTDs () {

    if (!document.getElementById("acceptTerms").checked) {
        window.alert ("Please read and agree to the terms and conditions of the WAP forum before downloading the DTDs");
        return;
    }

    // Kick off the downloading by creating a single objct
    downloaders.push (new DTDDownloader());

}

function DTDDownloader () {
    this.startDownload ();
}

DTDDownloader.prototype.startDownload = function () {
    var logger = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
    try {
        var file = getDTDDirectory();
        for (var ix = 0; ix < WMLDTDs.length; ix++) {
            var gotDtd = false;
            var knownDTD = WMLDTDs[ix];
            var shortDTD = knownDTD.match(/[^\/]+$/); // Get the part after the last '/' character
            var entries = file.directoryEntries;
            while (entries.hasMoreElements()) {
                var entry = entries.getNext();
                entry = entry.QueryInterface (Components.interfaces.nsIFile);
                if (entry.leafName == shortDTD) {
                    gotDtd = true;
                }
            }
            if (!gotDtd) {
                this.dtd = knownDTD;

                var that = this; // Copy of 'this' for use in closure
                this.DTDLoad = new XMLHttpRequest();
                this.DTDLoad.open ("GET", this.dtd);
                this.DTDLoad.onload  = function ()  { that.downloadOK() };
                this.DTDLoad.onerror = function ()  { that.downloadFailed() };
                this.DTDLoad.send(undefined);
                return;
            }
        }

        // If we reach this point then we have all the DTDs
        document.getElementById("downloadRequired")
            .setAttribute ("style", "display: none");
        document.getElementById("downloadNotRequired")
            .setAttribute ("style", "display: block");

    } catch (e) {
        logger.logStringMessage(e);
    }
}

DTDDownloader.prototype.downloadOK = function () {

    try {

        var file = getDTDDirectory();
        var shortDTD = this.dtd.match(/[^\/]+$/); // Strip off everything after the last '/'
        file.append (shortDTD);

        file.create(0,0644);

        var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
        var uri = ios.newFileURI (file);
        var channel = ios.newChannelFromURI (uri);
        var fs = Components.classes[ "@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        fs.init(file, 0x22, 00004, null); 

        fs.write (this.DTDLoad.responseText, this.DTDLoad.responseText.length);
        fs.close ();

    } catch (e) {
        var logger = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
        logger.logStringMessage ("DTDDownload.downloadOK: " + e);
        window.alert ("Received an error when trying to save DTDs\n(Do you have permission to write to your browser's res/dtd subdirectory?)");
    }

    // Start off a new download
    downloaders.push (new DTDDownloader());

}

DTDDownloader.prototype.downloadFailed = function () {
    window.alert ("There was an error fetching the DTD " + this.dtd);
}
